// SPDX-License-Identifier: GPL-3.0
//
// Copyright (C) 2019, Donald Dugger
//
import clock from "clock";
import { HeartRateSensor } from "heart-rate";
import { charger, battery } from "power";
import document from "document";
import * as C from "../common/constants.js";
import { logger, face, hilight } from "../common/utils";
import { utc, utc_off, tz_off, find_tz, numPad } from "../common/utils.js";
import { tz_data, hr_tumbler } from "../common/tz.js";
import * as prefs from "../common/prefs";

prefs.init();

let rotate;

face(document);

// Update the clock every second
clock.granularity = "seconds";

let hourHand = document.getElementById("hours");
let minHand = document.getElementById("mins");
let secHand = document.getElementById("secs");

let dayDate = document.getElementById("date");
let weekDay= document.getElementById("week");
const days = ["Su", "Mo", "Tu", "We", "Th", "Fr", "Sa"]

let cities1 = document.getElementById("tz_cities-1");
let cities2 = document.getElementById("tz_cities-2");

let tz1 = document.getElementById("tz1");
let tz1_index = 0;
tz1.text = "";

let tz2 = document.getElementById("tz2");
let tz2_index = -1;
tz2.text = "";
let time2 = document.getElementById("time2");

function set_rotation(r)
{

  rotate = r;
  let cc = document.getElementById("cc_btn");
  cc.image = ((r < 0) ? "ccw.png" : "cw.png");
  let min3 = document.getElementById("min-3");
  let min9 = document.getElementById("min-9");
  if (r < 0) {
    min3.text = "9";
    min9.text = "3";
  } else {
    min3.text = "3";
    min9.text = "9";
  }
}

let screens = [];
screens[C.SCR_FACE] = document.getElementById("analog_face");
screens[C.SCR_CFG0] = document.getElementById("config0");
screens[C.SCR_CFG1] = document.getElementById("config1");
screens[C.SCR_CFG2] = document.getElementById("config2");
screens[C.SCR_TZ1] = document.getElementById("custom_tz1");
screens[C.SCR_TZ2] = document.getElementById("custom_tz2");
let cfg_screen = C.SCR_CFG1;

let set_data = prefs.get();
tz1_index = set_data.tz1;
tz2_index = set_data.tz2;
let cities1_index = set_data.city1;
let cities2_index = set_data.city2;
set_rotation(set_data.rotation);

let bg = document.getElementById("click_bg");
bg.onclick = function(evt) {
  if (tz2_index >= 0) {
    let idx = tz1_index;
    tz1_index = tz2_index;
    tz2_index = idx;
  }
}

let batValue = document.getElementById("battery");

let bpmValue = document.getElementById("bpm");
let hrm = new HeartRateSensor();
hrm.onreading = function() {
  bpmValue.text = hrm.heartRate || "--";
  hrm.stop();
}

let url_text = document.getElementById("url_text");

let n = tz_data.length;
for (let i = 0; i < n; i++) {
  let tz_entry = screens[C.SCR_CFG1].getElementById("tz-" + i);
  if (tz_entry != null) {
    let t1_city = tz_entry.getElementById("touch");
    t1_city.onclick = evt => {
      hilight(screens[C.SCR_CFG1], tz1_index, "black");
      tz1_index = i;
      hilight(screens[C.SCR_CFG1], tz1_index, "grey");
    }
  }

  tz_entry = screens[C.SCR_CFG2].getElementById("tz-" + i);
  if (tz_entry != null) {
    let t2_city = tz_entry.getElementById("touch");
    t2_city.onclick = evt => {
      hilight(screens[C.SCR_CFG2], tz2_index, "black");
      tz2_index = i;
      hilight(screens[C.SCR_CFG2], tz2_index, "grey");
    }
  }
}

function sh_screen(idx)
{

  let max = screens.length;
  for (i = 0; i < max; i++)
    screens[i].style.display = "none";
  screens[idx].style.display = "inline";
}

//
// Button to bring up the detail page
//
let listBtn = document.getElementById("list_btn");
listBtn.onclick = function(evt) {
  hilight(screens[C.SCR_CFG1], tz1_index, "grey");
  if (tz2_index < 0)
    tz2_index = 0;
  hilight(screens[C.SCR_CFG2], tz2_index, "grey");
  cities1.value = cities1_index;
  cities2.value = cities2_index;
  sh_screen(cfg_screen);
}

function sh_main() {

  hilight(screens[C.SCR_CFG1], tz1_index, "black");
  hilight(screens[C.SCR_CFG2], tz2_index, "black");
  if (tz2_index == 0)
    tz2_index = -1;
  cities1_index = cities1.value;
  cities2_index = cities2.value;
  prefs.save(tz1_index, tz2_index,
             cities1_index, cities2_index,
             tz_data[32].off, tz_data[33].off,
             rotate);
  sh_screen(C.SCR_FACE);
}

let cc_btn = screens[C.SCR_CFG0].getElementById("cc_btn")
cc_btn.onclick = function(evt)
{

  set_rotation(-rotate);
}

let custom_btn1 = screens[C.SCR_CFG0].getElementById("btn_tz1");
custom_btn1.onclick = function(evt)
{   
      
  sh_screen(C.SCR_TZ1);
}

let custom_yes1 = screens[C.SCR_TZ1].getElementById("custom_yes");
custom_yes1.onclick = function(evt)
{   
      
  let hr_tumb = screens[C.SCR_TZ1].getElementById("hr_tumbler");
  let hr = hr_tumbler[hr_tumb.value];
  let min_tumb = screens[C.SCR_TZ1].getElementById("min_tumbler");
  let min = min_tumb.value * 15;
  tz_data[32].off = (hr * 60) + min;
  sh_screen(C.SCR_CFG0);
}

let custom_no1 = screens[C.SCR_TZ1].getElementById("custom_no");
custom_no1.onclick = function(evt)
{   
      
  sh_screen(C.SCR_CFG0);
}

let custom_btn2 = screens[C.SCR_CFG0].getElementById("btn_tz2");
custom_btn2.onclick = function(evt)
{   
      
  sh_screen(C.SCR_TZ2);
}

let custom_yes2 = screens[C.SCR_TZ2].getElementById("custom_yes");
custom_yes2.onclick = function(evt)
{   
      
  let hr_tumb = screens[C.SCR_TZ2].getElementById("hr_tumbler");
  let hr = hr_tumbler[hr_tumb.value];
  let min_tumb = screens[C.SCR_TZ2].getElementById("min_tumbler");
  let min = min_tumb.value * 15;
  tz_data[33].off = (hr * 60) + min;
  sh_screen(C.SCR_CFG0);
}

let custom_no2 = screens[C.SCR_TZ2].getElementById("custom_no");
custom_no2.onclick = function(evt)
{   
      
  sh_screen(C.SCR_CFG0);
}

//
// Buttons to go back to clock face
//
let clockBtn0 = screens[C.SCR_CFG0].getElementById("lr_btn");
clockBtn0.onclick = function(evt) {

  sh_main();
}
let clockBtn1 = screens[C.SCR_CFG1].getElementById("lr_btn");
clockBtn1.onclick = function(evt) {

  sh_main();
}
let clockBtn2 = screens[C.SCR_CFG2].getElementById("lr_btn");
clockBtn2.onclick = function(evt) {

  sh_main();
}

//
// Button to go to timezone 1 from cfg
//
let cfg_2_tz1 = screens[C.SCR_CFG0].getElementById("ur_btn");
cfg_2_tz1.onclick = function(evt) {
  cfg_screen = C.SCR_CFG1;
  sh_screen(cfg_screen);
}

//
// Button to go to config from tz1
//
let tz1_2_cfg = screens[C.SCR_CFG1].getElementById("ul_btn");
tz1_2_cfg.onclick = function(evt) {
  cfg_screen = C.SCR_CFG0;
  sh_screen(cfg_screen);
}

//
// Button to go to tz2 from tz1
//
let tz1_2_tz2 = screens[C.SCR_CFG1].getElementById("ur_btn");
tz1_2_tz2.onclick = function(evt) {
  cfg_screen = C.SCR_CFG2;
  sh_screen(cfg_screen);
}

//
// Button to go to tz1 from tz2
//
let tz1_2_tz2 = screens[C.SCR_CFG2].getElementById("ul_btn");
tz1_2_tz2.onclick = function(evt) {
  cfg_screen = C.SCR_CFG1;
  sh_screen(cfg_screen);
}

// Returns an angle (0-360) for the current hour in the day, including minutes
function hoursToAngle(hours, minutes) {
  let hourAngle = (360 / 12) * hours;
  let minAngle = (360 / 12 / 60) * minutes;
  return (hourAngle + minAngle) * rotate;
}

// Returns an angle (0-360) for minutes
function minutesToAngle(minutes) {
  return ((360 / 60) * minutes) * rotate;
}

// Returns an angle (0-360) for seconds
function secondsToAngle(seconds) {
  return ((360 / 60) * seconds) * rotate;
}

function updateBattery() {
  
  let lvl = Math.floor(battery.chargeLevel);
  batValue.text = lvl;
  if (lvl > 50)
    batValue.style.fill = "lime";
  else if (lvl > 25)
    batValue.style.fill = "yellow";
  else
    batValue.style.fill = "red";
}

function updateBpm() {
  
}

// Rotate the hands every tick
function updateClock() {

  let local = new Date();
  let tm1 = local;
  let tm2;
  if (tz1_index > 0 || tz2_index >= 0) {
    let off = utc_off(tm1.toString());
    let utc_time = utc(tm1, -off);
    if (tz1_index > 0)
      tm1 = utc(utc_time, tz_off(tz1_index, utc_time));
    if (tz2_index > 0)
      tm2 = utc(utc_time, tz_off(tz2_index, utc_time));
    else if (tz2_index == 0)
      tm2 = local;
  }
  let hours = tm1.getHours() % 12;
  let mins = tm1.getMinutes();
  let secs = tm1.getSeconds();

  hourHand.groupTransform.rotate.angle = hoursToAngle(hours, mins);
  minHand.groupTransform.rotate.angle = minutesToAngle(mins);
  secHand.groupTransform.rotate.angle = secondsToAngle(secs);

  if (tz1_index == 0)
    tz1.text = "";
  else
    tz1.text = find_tz(tz1_index, utc_time);

  let tz = "";
  let time = "";
  if (tz2_index >= 0) {
    tz = find_tz(tz2_index, utc_time);
    let hr = tm2.getHours();
    if (hr == 0)
      hr = 24;
    let pm = "";
    if (hr > 12) {
      hr -= 12;
      pm = " P";
    }
    let mn = tm2.getMinutes();
    time = numPad(hr, " ") + ":" + numPad(mn, "0") + pm;
  }
  tz2.text = tz;
  time2.text = time;

  dayDate.text = tm1.getDate();
  weekDay.text = days[tm1.getDay()];
  
  updateBattery();
  
  hrm.start();
}

// Update the clock every tick event
clock.ontick = () => updateClock();
